/******************************************************************

			  DCOMSCR.C

                Direct acces to the Screen

		   Author: FB.DRUILHE
		   Update: Dec 12, 1989

*******************************************************************/

#include <dos.h>
#include <conio.h>
#include <mem.h>

/*************** Direct Acces to the Screen ***********************/

#define CURSOR    0xF0
#define CURSOR_I  0xA0
#define NORMAL    0x07
#define INVERSE   0x70
#define SPACE     0x20

union REGS inregs,outregs;

int ch, cl; /* To save cursor */

/*-----------------------------------------------------------------*/
/*                    GENERAL FONCTIONS                            */
/*-----------------------------------------------------------------*/

cursfull() /* Full Cursor */
    {
    inregs.h.cl = 16; /* To hide cursor */
    inregs.h.ch = 0;
    inregs.h.ah = 1;
    int86(0x10,&inregs,&outregs);
    }

cursav() /* Save Cursor */
    {
    inregs.h.ah = 3;
    int86(0x10,&inregs,&outregs);
    ch=outregs.h.ch;
    cl=outregs.h.cl;
/*    printf("CURSOR: %u %u", ch, cl);
    delay(2000);
    inregs.h.cl = 16;
    inregs.h.ch = 8; */
    }

cursof() /* Cursor off */
    {
    inregs.h.cl = 0; /* To hide cursor */
    inregs.h.ch = 16;
    inregs.h.ah = 1;
    int86(0x10,&inregs,&outregs);
    }

curson() /* Cursor on ( Restore the mode ) */
    {
    inregs.h.cl = cl; /* To restore cursor */
    inregs.h.ch = ch;
    inregs.h.ah = 1;
    int86(0x10,&inregs,&outregs);
    }


/******************* MEMORY ASSIGMENT *************************************/

unsigned int base_screen;           /* Address of Screen for Direct Acces */

/* Far Pointer for Direct Screen Management */
char far *scr[3], far *scr_top[3], far *scr_end[3],
     far *scr_ll[3], far *scr_fl[3];

/* Attribute, cursor definition for the 3 differents Screen */
char sc_space[3], sc_cur[3], sc_atr[3];
size_t sc_size[3];
char *sc_emp[3];

char *empty_line_n[160];
char *empty_line_i[160];

void scroll_up( int n_sc )
/* Scroll Up one Screen */
{
#if   defined(__SMALL__)
	 movedata(base_screen, FP_OFF(scr_fl[n_sc]),
		  base_screen, FP_OFF(scr_top[n_sc]), sc_size[n_sc]);
	 movedata(_DS, (unsigned) sc_emp[n_sc],
		  base_screen, FP_OFF(scr_ll[n_sc]), 160);
#elif defined(__LARGE__)
	 memcpy(scr_top[n_sc], scr_fl[n_sc], sc_size[n_sc]);
	 memcpy(scr_ll[n_sc], sc_emp[n_sc], 160);
#elif
#error Direct Screen Management must be LARGE or SMALL
#endif
}

void clear_data_screen( int n_sc )
/* Clear One Screen */
{
	int i;
	/* Clear the whole screen */
	for (i=1; i<=24; i++)
	  scroll_up( n_sc );
	scr[n_sc]=scr_ll[n_sc];
	*scr[n_sc]++=sc_space[n_sc];
	*scr[n_sc]=sc_cur[n_sc];
}

void init_screen()
/* Define Screens Parameters */
{
	struct text_info tx_info;
	int i;
	char *sc;

        gettextinfo(&tx_info);
	clrscr();

	sc = (char *) &empty_line_n[0];
	for (i=0;i<160;i+=2)
	  {
	  *sc++=SPACE;
	  *sc++=NORMAL;
	  }
	sc = (char *) &empty_line_i[0];
	for (i=0;i<160;i+=2)
	  {
	  *sc++=SPACE;
	  *sc++=INVERSE;
	  }

	if (tx_info.currmode<=3) base_screen = 0xB800;
	else base_screen  = 0xB000;

#if defined(DEBUG)
	printf("TextMode : %u  Adr %x\n", tx_info.currmode, base_screen);
#endif
	/* Screen 0 is 1 to 24 lines, Normal Video */
	scr_top[0] = MK_FP( base_screen, 160*0           );
	scr_end[0] = MK_FP( base_screen, (160*24)-2      );
	scr_fl[0]  = MK_FP( base_screen, 160*(0+1)       );
	scr_ll[0]  = MK_FP( base_screen, 160*(24-1)      );
	scr[0]     = scr_ll[0];
	sc_space[0]= SPACE;
	sc_cur[0]  = CURSOR;
	sc_atr[0]  = NORMAL;
	sc_size[0] = 160*23;
	sc_emp[0]  = (char *) &empty_line_n[0];

	/* Screen 1 is 12 to 24 lines, Inverse Video */
	scr_top[1] = MK_FP( base_screen, 160*12          );
	scr_end[1] = MK_FP( base_screen, (160*24)-2      );
	scr_fl[1]  = MK_FP( base_screen, 160*(12+1)      );
	scr_ll[1]  = MK_FP( base_screen, 160*(24-1)      );
	scr[1]     = scr_ll[1];
	sc_space[1]= SPACE;
	sc_cur[1]  = CURSOR_I;
	sc_atr[1]  = INVERSE;
	sc_size[1] = 160*11;
	sc_emp[1]  = (char *) &empty_line_i[0];

	/* Screen 2 is 1 to 12 lines, Normal Video */
	scr_top[2] = MK_FP( base_screen, 160*0           );
	scr_end[2] = MK_FP( base_screen, (160*12)-2      );
	scr_fl[2]  = MK_FP( base_screen, 160*(0+1)       );
	scr_ll[2]  = MK_FP( base_screen, 160*(12-1)      );
	scr[2]     = scr_ll[2];
	sc_space[2]= SPACE;
	sc_cur[2]  = CURSOR;
	sc_atr[2]  = NORMAL;
	sc_size[2] = 160*11;
	sc_emp[2]  = (char *) &empty_line_n[0];
}

void set_attribut_0( int i)
{
	if (i==1)
	  {
	  sc_space[0]= SPACE;
	  sc_cur[0]  = CURSOR_I;
	  sc_atr[0]  = INVERSE;
	  sc_emp[0]  = (char *) &empty_line_i[0];
	  }
	else
	  {
	  sc_space[0]= SPACE;
	  sc_cur[0]  = CURSOR;
	  sc_atr[0]  = NORMAL;
	  sc_emp[0]  = (char *) &empty_line_n[0];
	  }
}

int put_screen( char c, int n_screen)
{
       /* Put normal Atribut for Cursor */
       *scr[n_screen] = sc_atr[n_screen];
       if (c==8)
	 { /* BS */
	 scr[n_screen]--; /* Point on Character */
	 if (scr[n_screen]!=scr_ll[n_screen]) scr[n_screen]-=2;
	 scr[n_screen]++; /* Point on Attribut */
	 *scr[n_screen]=sc_cur[n_screen];
	 return(0);
	 }

       else if (c==13)
	 { /* CR */
	 scr[n_screen]=scr_ll[n_screen];
	 *++scr[n_screen]=sc_cur[n_screen];
	 return(0);
	 }

       else if (c==10)
	 { /* LF */
	 scroll_up( n_screen );
	 /* LF generate also a CR (for File)   */
	 scr[n_screen]=scr_ll[n_screen];
	 *++scr[n_screen]=sc_cur[n_screen];
	 return(0);
	 }

       /* Write Caracter */

       *--scr[n_screen]=c /* Character */;
       if (scr[n_screen]==scr_end[n_screen])
	 {
	 scroll_up( n_screen );
	 scr[n_screen]=scr_ll[n_screen];
	 }
       else scr[n_screen]+=2 /* next Char */;
       *++scr[n_screen]=sc_cur[n_screen];
       return(0);
}
